import java.io.*;

public class BookGen { 
    public static void main(String[] args) throws Exception { 
        if (args.length < 2) { 
            System.err.println("Usage: java BookGen BOOK-NAME BOOK-TEXT [BOOK-CONTENT]");
        }
        String bookName = args[0];
        String bookText = args[1];
        PrintStream f = new PrintStream(new FileOutputStream("MANIFEST.MF"));
        f.println("MIDlet-1: " + bookName + ", /book.png, BookReader");
        f.println("MIDlet-Name: " + bookName);
        f.println("MIDlet-Vendor: Smart Mobile Technologies");
        f.println("MIDlet-Version: 1.0");
        f.println("MicroEdition-Configuration: CLDC-1.0");
        f.println("MicroEdition-Profile: MIDP-1.0");
        f.close();

        InputStream in = new FileInputStream(bookText);
        byte[] buf = new byte[in.available()];
        int len = in.read(buf);
        if (len != buf.length) { 
            throw new Error("Failed to read file");
        }
        int src = 0, dst = 0;
        boolean html = buf[0] == '<';             
        while (src < len) { 
            byte b = buf[src++];
            if (html && b == '<') { 
                if (src+2 < len && buf[src] == '!' && buf[src+1] == '-' && buf[src+2] == '-') {
                    src += 2;
                    do { 
                        src += 1;
                    } while (src+3 < len && !(buf[src] == '-' &&  buf[src+1] == '-' &&  buf[src+2] == '>'));
                    src += 3;
                } else if (src+1 < len && (buf[src] == 'p' || buf[src] == 'P') && (buf[src+1] == ' ' || buf[src+1] == '>')) { 
                    buf[dst++] = '\n';
                    while (src < len && buf[src++] != '>');
                } else { 
                    int start = src;
                    do { 
                        if (src == len) { 
                            src = start;
                            break;
                        }
                    } while (buf[src++] != '>');
                }
            } else if (html && b == '&') { 
                if (src+3 < len && buf[src] == 'q' && buf[src+1] == 'u' && buf[src+2] == 'o' && buf[src+3] == 't') { 
                    buf[dst++] = '"';
                    src += 5;
                } else if (src+3 < len && buf[src] == 'n' && buf[src+1] == 'b' && buf[src+2] == 's' && buf[src+3] == 'p') { 
                    buf[dst++] = '"';
                    src += 5;
                } else if (src+2 < len && buf[src] == 'a' && buf[src+1] == 'm' && buf[src+2] == 'p') { 
                    buf[dst++] = '&';
                    src += 4;
                } else if (src+1 < len && buf[src] == 'l' && buf[src+1] == 't') { 
                    buf[dst++] = '<';
                    src += 3;
                } else if (src+1 < len && buf[src] == 'g' && buf[src+1] == 't') { 
                    buf[dst++] = '>';
                    src += 3;
                } else if (src < len && buf[src] == '#') {
                    if (Character.isSpace((char)buf[++src])) { 
                        src += 1;
                    }
                    if (buf[src] == 'x') {                             
                        int start = ++src;
                        while (++src < len && Character.isLetterOrDigit((char)buf[src]));
                        buf[dst++] = (byte)Integer.parseInt(new String(buf, start, src-start).trim(), 16);
                    } else { 
                        int start = src;
                        while (++src < len && Character.isDigit((char)buf[src]));
                        buf[dst++] = (byte)Integer.parseInt(new String(buf, start, src-start).trim());
                    }                            
                    if (src < len && buf[src] == ';') { 
                        src += 1;
                    }
                } else { 
                    buf[dst++] = b;
                }
            } else if (b != '\r' && b >= (byte)0xC0) { 
                buf[dst++] = b;
            }
        }
        OutputStream out = new FileOutputStream("res/book.txt");
        out.write(buf, 0, dst);
        out.close();
        
        if (args.length > 2) { 
            String text = new String(buf, 0, dst);
            String item;
            BufferedReader cin = new BufferedReader(new FileReader(args[2]));
            DataOutputStream cout = new DataOutputStream(new FileOutputStream("res/content"));
            int prevPos = 0;
            while ((item = cin.readLine()) != null) { 
                int pos = text.indexOf(item, prevPos);
                if (pos < 0) { 
                    System.out.println("Failed to locate in text item " + item);
                } else { 
                    cout.writeInt(pos);
                    cout.writeByte(item.length());
                    cout.write(item.getBytes());          
                    prevPos = pos;
                }
            }
            cout.close();
            cin.close();
        } else {
            File oldContent = new File("res/content");
            oldContent.delete();
        }

        DataOutputStream cfg = new DataOutputStream(new FileOutputStream("res/config"));
        cfg.writeInt(dst);
        cfg.close();

        Runtime rt = Runtime.getRuntime();
        Process proc = rt.exec("jar cfm " + bookName + ".jar MANIFEST.MF -C classes . -C res .");
        proc.waitFor();
        
        File jar = new File(bookName + ".jar");

        f = new PrintStream(new FileOutputStream(bookName + ".jad"));
        f.println("MIDlet-1: " + bookName + ", /book.png, BookReader");
        f.println("MIDlet-Jar-Size: " + jar.length());
        f.println("MIDlet-Jar-URL: " + bookName + ".jar");
        f.println("MIDlet-Name: " + bookName);
        f.println("MIDlet-Vendor: Smart Mobile Technologies");
        f.println("MIDlet-Version: 1.0");
        f.println("MicroEdition-Configuration: CLDC-1.0");
        f.println("MicroEdition-Profile: MIDP-1.0");
        f.close();

        proc = rt.exec("zip \\public_html\\" + bookName + ".zip " + bookName + ".jad " + bookName + ".jar");
        proc.waitFor();
        
        System.out.println("Book '" + bookName + "' is generated");
    }
}