/*
 * Created on 26.01.2005
 */
import javax.microedition.lcdui.*;
import java.io.*;

public class ImageFont implements IFont  
{
    private int [] _props;
    private byte [] _width;
    private String _fontName;
    private boolean isLoaded;
    private Image _buf;
    private int _count, _height;

    public ImageFont (String fontName)
    {
        _fontName = fontName;
        loadFont ();
    }
    
    private int getWord (InputStream stream) throws IOException
    {
        int lo = stream.read ();
        int hi = stream.read ();
        if (lo < 0 || hi < 0) return -1;
        return hi * 256 + lo;
    }

    public void loadFont ()
    {
        if (isLoaded)
            clear ();
        try
        {
            InputStream in = this.getClass ().getResourceAsStream (_fontName + ".props");
            //    -  

            System.out.println("Open properties");
            _count = getWord (in); 
            _props = new int [_count];
            _width = new byte [_count];
            //      2       
            _props[0] = getWord (in);
            for (int i = 1; i < _count; i++) {
                _props[i] = getWord(in);
                _width[i-1] = (byte)(_props[i] - _props[i-1]);
            }
            _buf = Image.createImage(_fontName + ".png");
            _height = _buf.getHeight ();
            _width [_count-1] = (byte) (_buf.getWidth () - _props [_count - 1]);
            isLoaded = true;
            // System.out.println ("Font loaded");
        } catch (Exception e) {
            System.out.println("createImage throws exception " + e);
            e.printStackTrace();
            clear ();
            return;
        }
    }
    
    public void drawString (Graphics g, int x, int y, String s)
    {
        if (!isLoaded) 
        {
            g.drawString ("Font isn't loaded!", 0, 0, 0);
            System.out.println ("Can't draw string: Font isn't loaded");
            return;
        }
        int clipx = g.getClipX ();
        int clipy = g.getClipY ();
        int clipw = g.getClipWidth ();
        int cliph = g.getClipHeight ();
        int offs = x;
        for (int i = 0; i < s.length (); i ++)
        {
            int ch = s.charAt(i);
            if (Locale.RUSSIAN) { 
                ch -= 1;
                ch = ch == 0x400 ? 0xa7 : ch == 0x450 ? 0xb7 : ch;
                ch = ch > 0x400 ? ch - 0x350 : ch;
            }
            if (ch < 0 || ch >= _count)
            {
                //System.out.println ("Char " + i + "(" + ch + ") in string " + s + " is invalid");
                continue;
            }
            int width = _width [ch];
            if (ch > 31)
            {
                int imgx = _props [ch];
                g.setClip (offs, y, width, _height);
                g.drawImage (_buf, offs - imgx, y, Graphics.LEFT | Graphics.TOP);
            }
            offs += width;
        }
        g.setClip (clipx, clipy, clipw, cliph);
    }
    
    public int charWidth (char ch)
    {
        if (!isLoaded)
        {
            System.out.println ("Can't get char width: Font isn't loaded");
            return 0;
        }
        if (Locale.RUSSIAN) { 
            ch = (char) (ch == 0x401 ? 0xa8 : ch == 0x451 ? 0xb8 : ch);
            ch = (char) (ch > 0x400 ? ch == 0x451 ? 0xb8 : ch == 0x401 ? 0xa8 : ch - 0x351 : ch - 1);
        }
        if (ch < 0 || ch >= _count)
        {
            // System.out.println ("Char code " + (int) ch + " is invalid");
            return 0; 
        }
        //System.out.println("Width of character "+ ch + " is " + _width[ch]);
        return _width [ch];
    }
    
    public int getHeight ()
    {
        return _height;
    }

    private void clear ()
    {
        if (!isLoaded) return;
        _fontName = "";
        _buf = null;
        _props = null;
        _width = null;
        _height = 0;
    }
}